require "xmlrpc/client"
require "digest/sha2"
require "yaml"
module Inwx
  DEFAULT_WEB_IP = "0.0.0.0"
  DEFAULT_NAMESERVER_TYPE = "MASTER"
  DEFAULT_NAMESERVER_LIST = ["ns.danielthal.de", "ns2.danielthal.de", "ns3.danielthal.de", "ns4.danielthal.de", "ns5.danielthal.de"]
  DEFAULT_MAILSERVER_IP = "0.0.0.0"
  DEFAULT_SOA_MAIL = "hostmaster@inwx.de"

  class Nameserver
    attr_accessor :api_client

    def initialize(user=false, pass=false, address="api.domrobot.com", lang="en")
      self.api_client = DomainRobot.new(address, user, pass, lang)
    end

    def user=(u)
      self.api_client.username=u
    end

    def pass=(p)
      self.api_client.password=p
    end

    def address=(a)
      self.api_client.address=(a)
    end

    def add_domain(domain_name, send_params={}) # IS WORKING CORRECT
      params = {:domain => domain_name, :type => DEFAULT_NAMESERVER_TYPE, :ns => DEFAULT_NAMESERVER_LIST, :web => DEFAULT_WEB_IP, :mail => DEFAULT_MAILSERVER_IP, :soaEmail => DEFAULT_SOA_MAIL}
      params.merge! send_params
      response = self.api_client.call("nameserver", "create", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end

    def delete_domain(domain_name, roId) # IS WORKING CORRECT
      params = {:domain => domain_name, :roId => roId}
      response = self.api_client.call("nameserver", "delete", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end
    
    def list_domains(page=0, pagelimit=0) # IS WORKING CORRECT
      params = {:page => page, :pagelimit => pagelimit}
      response = self.api_client.call("nameserver", "list", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end

    def search_domain(domain, page=0, pagelimit=0) # IS WORKING CORRECT
      params = {:domain => domain, :page => page, :pagelimit => pagelimit}
      response = self.api_client.call("nameserver", "list", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end

    def info_domain(roId) # IS WORKING CORRECT
      params = {:roId => roId}
      response = self.api_client.call("nameserver", "info", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end

    def create_domain_record(roId, type, content, send_params={}) # IS WORKING CORRECT
      params = {:roId => roId, :type => type, :content => content}
      params.merge! send_params
      response = self.api_client.call("nameserver", "createRecord", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response] 
    end

    def delete_domain_record(id) # IS WORKING CORRECT
      params = {:id => id}
      response = self.api_client.call("nameserver", "deleteRecord", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end

    def update_domain_record(id, send_params={}) # IS WORKING CORRECT
      params = {:id => id}
      params.merge! send_params
      response = self.api_client.call("nameserver", "updateRecord", params)
      return [true, response] if response["code"] == 1000 or response["code"] == 1001
      [false, response]
    end
  end


  class DomainRobot
    attr_accessor :address, :username, :password, :language, :secure
    
    def initialize(address, username = false, password = false, language = 'en', secure = false)
      @address = address
      @username = username
      @password = password
      @language = language
      @secure = secure
    end
    
    def call(object, method, params = {})
      if self.secure 
        # Get a one-time-"random" nonce
        nonce = Time.now.to_f
        
        # Generate hash from nonce and password.
        hash = Digest::SHA2.new(bitlen = 256) << nonce.to_s + self.password.to_s
        
        # Merge both into params
        params.merge! :pass => hash.to_s        
      else               
        params.merge! :pass => self.password
      end
      
      # Merge username into params
      params.merge! :user => self.username
     
      # Create a new client instance
      client = XMLRPC::Client.new(self.address, "/xmlrpc/", "443", nil, nil, nil, nil, true, 900)
      
      # Call the remote method
      client.call(object+"."+method, params)
    end
  end
end
